/*
 * Implementao de um filtro em grade multi-cannal como os coeficiente opcionalmente ligados a outra implementao.
 *
 * As entradas so:
 * - u(n), reference signals
 * - d(n), desired signals
 *
 * As saidas so:
 * d^(n|Un)
 * o que  clculado basedo no erro de predio  priori:
 * d^(n|Un) = d(n) - gamma(n-1)*alfa1(n)
 *
 * Paulo Lopes 17/12/2001
 */

#pragma once

#include <math.h>
#include <memory.h> 
#include <assert.h>
#include <stdio.h>

/* Lattice arguments
 *
 * - Number of reference Signals
 * - Number of output Signals
 * - Lattice Order
 * - Inicialization
 */
#include "Tensor.h"

typedef val_type MyData;

class McLatticePr {
private:
	void IniLatticePr(int inSize, int outSize, int order);
public:
	bool link;
	MyData *eta1p;
	MyData *psi1p;
	MyData *alfa1p;
	
	MyData *eta0p;
	MyData *psi0p;
	MyData *GammaF0p;
	MyData *GammaB0p;
	MyData *alfa0p;
	MyData *kappa0p;
	MyData *gamma0p;

	int order;
	int inSize;
	int inSizeSq;
	int outSize;
	int outSizeSq;

	McLatticePr(int inSize, int outSize, int order);
	McLatticePr(int inSize, int outSize, int order,  
		MyData *GammaF0, MyData *GammaB0, MyData *kappa0, MyData *gamma0);
	~McLatticePr(void);

	// Update function
	void Update(val_type *in);

	// Output function
	void Output(val_type *out);

	// Funes auxiliares
	MyData &eta1(int m,int i)				
	{ assert(m<=order && i<inSize); return eta1p[m*inSize+i]; }
	MyData &psi1(int m,int i)				
	{ assert(m<=order && i<inSize); return psi1p[m*inSize+i]; }
	MyData &alfa1(int m,int i)
	{ assert(m<=order && i<outSize); return alfa1p[m*outSize+i]; }
	MyData &eta0(int m,int i)				
	{ assert(m<=order && i<inSize); return eta0p[m*inSize+i]; }
	MyData &psi0(int m,int i)				
	{ assert(m<=order && i<inSize); return psi0p[m*inSize+i]; }
	MyData &GammaF0(int m, int i, int j)
	{ assert(m<=order && i<inSize && j<inSize); return GammaF0p[m*inSizeSq+j*inSize+i]; }
	MyData &GammaB0(int m, int i, int j)		
	{ assert(m<=order && i<inSize && j<inSize); return GammaB0p[m*inSizeSq+j*inSize+i]; }
	MyData &gamma0(int m)					
	{ assert(m<=order); return gamma0p[m]; }
	MyData &alfa0(int m,int i)				
	{ assert(m<=order && i<outSize); return alfa0p[m*outSize+i]; }
	MyData &kappa0(int m, int i, int j)		
	{ 
		assert(m<=order && i<outSize && j<inSize);
		return kappa0p[m*outSize*inSize+j*outSize+i];
	}
};
